using System;
using Microsoft.SPOT;

using System.Threading;

using GHIElectronics.NETMF.Hardware;
using GHIElectronics.NETMF.USBHost;

namespace WatchRobot
{
    public class Watch
    {
        USBH_SerialUSB watchComs = null;

        byte[] startAccessPoint = new byte[] { 0xFF, 0x07, 0x03 };
        byte[] readAccel = new byte[] { 0xFF, 0x08, 0x07, 0x00, 0x00, 0x00, 0x00 };

        Thread watchControlThread = null;

        byte[] buffer = new byte[100];

        public Watch()
        {
            USBHostController.DeviceConnectedEvent +=
                new USBH_DeviceConnectionEventHandler(USBHostController_DeviceConnectedEvent);
            USBHostController.DeviceDisconnectedEvent +=
                new USBH_DeviceConnectionEventHandler(USBHostController_DeviceDisconnectedEvent);
        }

        void USBHostController_DeviceDisconnectedEvent(GHIElectronics.NETMF.USBHost.USBH_Device device)
        {
            if (watchControlThread != null)
            {
                watchControlThread.Abort();
                watchControlThread = null;
                watchComs = null;
                watchPos.SetToZero();
                successfulReading = false;
            }
        }

        void USBHostController_DeviceConnectedEvent(USBH_Device device)
        {
            watchComs = new USBH_SerialUSB(
                device,
                115200,
                System.IO.Ports.Parity.None,
                8,
                System.IO.Ports.StopBits.One);

            watchControlThread = new Thread(new ThreadStart(watchControl));
            watchControlThread.Start();
        }

        public struct Vector3
        {
            public sbyte X;
            public sbyte Y;
            public sbyte Z;

            public void SetToZero()
            {
                X = 0;
                Y = 0;
                Z = 0;
            }

            public override string ToString()
            {
                return "X:" + X.ToString() +
                    " Y:" + Y.ToString() +
                    " Z:" + Z.ToString();
            }
        }

        private Vector3 watchPos;

        public Vector3 WatchPos
        {
            get
            {
                Monitor.Enter(watchLock);
                Vector3 result = watchPos;
                Monitor.Exit(watchLock);
                return result;
            }
        }

        private DateTime lastReading;

        public DateTime LastReading
        {
            get
            {
                Monitor.Enter(watchLock);
                DateTime result = lastReading;
                Monitor.Exit(watchLock);
                return result;
            }
        }

        object watchLock = new object();

        private bool successfulReading = false;

        public bool WatchActive
        {
            get
            {
                return successfulReading;
            }
        }

        private void watchControl()
        {
            watchComs.Write(startAccessPoint, 0, startAccessPoint.Length);

            while (watchComs != null)
            {
                try
                {
                    watchComs.Write(readAccel, 0, readAccel.Length);

                    int bytesRead = watchComs.Read(buffer, 0, buffer.Length);

                    if (buffer[0] == 0xff &&
                        buffer[1] == 0x06 &&
                        buffer[2] == 0x07 &&
                        buffer[3] == 0x01)
                    {
                        Monitor.Enter(watchLock);
                        watchPos.X = (sbyte)buffer[4];
                        watchPos.Y = (sbyte)buffer[5];
                        watchPos.Z = (sbyte)buffer[6];
                        lastReading = DateTime.Now;
                        successfulReading = true;
                        Monitor.Exit(watchLock);
                        if (NewReading != null)
                        {
                            // Fire off a thread to run any methdos bound to the
                            // new reading event
                            Thread readingThread = new Thread(new ThreadStart(newReadingEvent));
                            readingThread.Start();
                        }
                    }
                    else
                    {
                        successfulReading = false;
                    }
                }
                catch
                {
                    // just ignore the errors for now
                }
            }
        }

        public delegate void NewWatchReading(Watch watch);

        private void newReadingEvent()
        {
            NewReading(this);
        }

        public NewWatchReading NewReading = null;
    }
}
